{% extends "base.html" %}
{% load i18n %}

{% block layout %}
{% include "partials/nav_accounts.html" %}

<div class="container-fluid">
    <div class="hero-unit">
        <h1>{% trans 'Internal server error!' %}</h1>
        <p>{% trans "Something went terribly wrong.  We're very sorry and will fix this problem as soon as possible." %}</p>
        <p>
            <a class="btn btn-primary btn-large" onClick="javascript:history.go(-1);">{% trans 'Go back to previous page' %}</a>
			<a class="btn btn-primary btn-large" href="/">{% trans "Homepage" %}</a>
        </p>
    </div>
</div>
{% endblock %}